# マルコフ基底 
markov_b <- matrix(nrow = 6, ncol = 6)
markov_b[1,] <- c(0,-1, 1, 0, 1,-1) 
markov_b[2,] <- c(1,-1, 0,-1, 1, 0)
markov_b[3,] <- c(1, 0,-1,-1, 0, 1)
markov_b[4,] <- c(0, 1,-1, 0,-1, 1) 
markov_b[5,] <- c(-1, 1, 0, 1,-1, 0)
markov_b[6,] <- c(-1, 0, 1, 1, 0,-1)

# p(y + z)/p(y) の計算 (z はマルコフ基底のいずれか)
# p(y) = 1/Z 1/(y[1]! ... y[length(y)]!)
prob_r <- function(y, z) {
	len <- length(y)
	r <- 1
	# マルコフ基底の成分が 1, 0, -1 の場合で場合分けして計算
	# マルコフ基底の成分で 1, 0, -1 以外が出る場合は使えないことに注意
	for (i in 1:len) {
		if (z[i] == 1) {
			r <- r / (y[i] + 1)
		} else if (z[i] == -1) {
			r <- r * y[i]
		}
	}
	return(r)
}

# 2x3 分割表 MCMC 
# 引数
# n      : 取り出す数
# burnin : 最初からいくら捨てるか
# state  : 最初の状態 
c2x3mcmc <- function(n = 10000, burnin = 10000, state = c(0,5,0,5,0,10)) {
	sum <- 0.0

	select <- sample(c(1:6), n, replace=TRUE) # マルコフ基底 markov_b のどれ(1 or ... or 8)を選ぶか
	rr <- runif(n)
	for (i in 1:burnin) {
		index <- select[i]
		r <- prob_r(state, markov_b[index, ])	
		if (r > rr[i]) {
			state <- state + markov_b[index,]
		}
	}

	select <- sample(c(1:6), n, replace=TRUE) # マルコフ基底 markov_b のどれ(1 or ... or 8)を選ぶか
	rr <- runif(n)
	for (i in 1:n) {
		sum <- sum + 2 * state[1] + state[2]
		index <- select[i]
		r <- prob_r(state, markov_b[index, ])	
		if (r > rr[i]) {
			state <- state + markov_b[index,]
		}
		print(state)  # サンプル値の表示
	}
	ave <- sum / n
	cat("平均\n")
	print(ave)
}

cat("c2x3mcmc(n, burnin, 初期値) で実行\n")
cat("e.g. c2x3mcmc(10000, 10000, c(0,5,0,5,0,10))\n")
