#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define M 82 
#define N 16 

void v_add(int *v1, int *v2, int n)
{
	int i;

	for (i = 0; i < n; i++)
		v1[i] += v2[i];
}

void v_show(int *v, int n)
{
	int i;

	for (i = 0; i < n; i++)
		printf("%2d ", v[i]);	
	printf("\n");
}

double prob_r(int *y, int *z, int n)
{
	int i, t;	
	double r;

	r = 1.0;
	for (i = 0; i < n; i++)
		if (z[i] > 0) {
			for (t = 1; t <= z[i]; t++)
				r /= (y[i] + t);
		} else if (z[i] < 0) {
			for (t = 0; t > z[i]; t--)
				r *= (y[i] + t);
		}
	return r;
}

double chisq(int *y, double *yy, int n)
{
        int i;
        double x = 0.0;

        for (i = 0; i < n; i++)
                x += (y[i]-yy[i])*(y[i]-yy[i])/yy[i];
        return x;
}

double *fitted_v(double *rs, double *cs, int sum, int m)
{
        int i, j;
        double *v;

        v = (double *)malloc(sizeof(double) * m * m);
        for (i = 0; i < m; i++)
                for (j = 0; j < m; j++)
                        v[m * i + j] = rs[i] * cs[j]/sum;
        return v;
}

double likelihood_ratio(int *y, double *mu, int n)
{
	int i;
	double x = 0.0;

	for (i = 0; i < n; i++)
		x += y[i] * log(y[i]/mu[i]);
	return 2 * x;
}

int main(int argc, char *argv[])
{
	int i, j;
	int seed = 0;
	int n = 10000;
	int burnin = n;
	double r;
	double rr;
	int markov_b[M][N] = {
{ 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,-1,-1,-1,-1, 1, 1 },
{ 0, 0, 0, 0, 1,-1,-1, 1, 0,-2, 0, 2, 1, 1,-1,-1 },
{ 0, 0, 0, 0, 1,-1,-1, 1, 1,-1,-1, 1, 0, 0, 0, 0 },
{ 0, 0, 0, 0, 1,-1,-1, 1, 2, 0,-2, 0,-1,-1, 1, 1 },
{ 0, 0, 1,-1, 0, 0, 1,-1,-2, 0, 1, 1, 0, 2,-1,-1 },
{ 0, 0, 1,-1, 0, 0, 1,-1,-1, 1, 0, 0,-1, 1, 0, 0 },
{ 0, 0, 1,-1, 0, 0, 1,-1, 0, 2,-1,-1,-2, 0, 1, 1 },
{ 0, 0, 1,-1, 1,-1, 0, 0, 0, 0,-1, 1,-1, 1, 0, 0 },
{ 0, 1,-1, 0,-2, 1, 1, 0,-1, 0, 2,-1, 1, 0, 0,-1 },
{ 0, 1,-1, 0,-1, 0, 0, 1, 0,-1, 1, 0, 1, 0, 0,-1 },
{ 0, 1,-1, 0, 0,-1,-1, 2, 1,-2, 0, 1, 1, 0, 0,-1 },
{ 0, 1, 0,-1,-1, 0, 1, 0,-1, 0, 1, 0, 0, 1, 0,-1 },
{ 0, 1, 0,-1,-1, 0, 1, 0, 0, 1, 0,-1,-1, 0, 1, 0 },
{ 0, 1, 0,-1, 0,-1, 0, 1, 0,-1, 0, 1, 0, 1, 0,-1 },
{ 0, 1, 0,-1, 0,-1, 0, 1, 1, 0,-1, 0,-1, 0, 1, 0 },
{ 0, 1, 1,-2, 0,-1, 1, 0,-1, 0, 0, 1,-1, 2, 0,-1 },
{ 0, 2,-1,-1,-2, 0, 1, 1, 0, 0, 1,-1, 0, 0, 1,-1 },
{ 0, 2, 0,-2,-1,-1, 1, 1, 0, 0, 0, 0,-1, 1, 1,-1 },
{ 1,-2, 0, 1, 1, 0, 0,-1, 0, 1,-1, 0, 0,-1,-1, 2 },
{ 1,-1,-1, 1, 0, 0, 0, 0,-1,-1, 1, 1, 2, 0,-2, 0 },
{ 1,-1,-1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1,-1,-1, 1 },
{ 1,-1,-1, 1, 0, 0, 0, 0, 1, 1,-1,-1, 0,-2, 0, 2 },
{ 1,-1, 0, 0, 1,-1, 0, 0,-1,-1, 0, 2, 1, 1,-2, 0 },
{ 1,-1, 0, 0, 1,-1, 0, 0, 0, 0,-1, 1, 0, 0,-1, 1 },
{ 1,-1, 0, 0, 1,-1, 0, 0, 1, 1,-2, 0,-1,-1, 0, 2 },
{ 1, 0,-2, 1,-1, 0, 0, 1, 0,-1, 1, 0, 2,-1,-1, 0 },
{ 1, 0,-1, 0,-1, 0, 1, 0,-1, 0, 1, 0, 1, 0,-1, 0 },
{ 1, 0,-1, 0, 0,-1, 0, 1, 0,-1, 0, 1, 1, 0,-1, 0 },
{ 1, 0,-1, 0, 0,-1, 0, 1, 1, 0,-1, 0, 0,-1, 0, 1 },
{ 1, 0, 0,-1,-1, 0, 2,-1,-2, 1, 1, 0, 0, 1,-1, 0 },
{ 1, 0, 0,-1, 0,-1, 1, 0,-1, 0, 0, 1, 0, 1,-1, 0 },
{ 1, 0, 0,-1, 0,-1, 1, 0, 0, 1,-1, 0,-1, 0, 0, 1 },
{ 1, 0, 0,-1, 1,-2, 0, 1, 0,-1,-1, 2, 0, 1,-1, 0 },
{ 1, 1,-2, 0,-1,-1, 0, 2, 1,-1, 0, 0, 1,-1, 0, 0 },
{ 1, 1,-1,-1,-2, 0, 2, 0,-1, 1, 1,-1, 0, 0, 0, 0 },
{ 1, 1,-1,-1,-1,-1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0 },
{ 1, 1,-1,-1, 0,-2, 0, 2, 1,-1,-1, 1, 0, 0, 0, 0 },
{ 1, 1, 0,-2,-1,-1, 2, 0,-1, 1, 0, 0,-1, 1, 0, 0 },
{ 2,-1,-1, 0, 0,-1, 1, 0,-1, 0, 0, 1, 1, 0,-2, 1 },
{ 2, 0,-2, 0,-1,-1, 1, 1, 0, 0, 0, 0, 1,-1,-1, 1 },
{ 2, 0,-1,-1, 0,-2, 1, 1, 0, 0,-1, 1, 0, 0,-1, 1 },

{ 0, 0, 0, 0, 0, 0, 0, 0,-1,-1, 1, 1, 1, 1,-1,-1 },
{ 0, 0, 0, 0,-1, 1, 1,-1, 0, 2, 0,-2,-1,-1, 1, 1 },
{ 0, 0, 0, 0,-1, 1, 1,-1,-1, 1, 1,-1, 0, 0, 0, 0 },
{ 0, 0, 0, 0,-1, 1, 1,-1,-2, 0, 2, 0, 1, 1,-1,-1 },
{ 0, 0,-1, 1, 0, 0,-1, 1, 2, 0,-1,-1, 0,-2, 1, 1 },
{ 0, 0,-1, 1, 0, 0,-1, 1, 1,-1, 0, 0, 1,-1, 0, 0 },
{ 0, 0,-1, 1, 0, 0,-1, 1, 0,-2, 1, 1, 2, 0,-1,-1 },
{ 0, 0,-1, 1,-1, 1, 0, 0, 0, 0, 1,-1, 1,-1, 0, 0 },
{ 0,-1, 1, 0, 2,-1,-1, 0, 1, 0,-2, 1,-1, 0, 0, 1 },
{ 0,-1, 1, 0, 1, 0, 0,-1, 0, 1,-1, 0,-1, 0, 0, 1 },
{ 0,-1, 1, 0, 0, 1, 1,-2,-1, 2, 0,-1,-1, 0, 0, 1 },
{ 0,-1, 0, 1, 1, 0,-1, 0, 1, 0,-1, 0, 0,-1, 0, 1 },
{ 0,-1, 0, 1, 1, 0,-1, 0, 0,-1, 0, 1, 1, 0,-1, 0 },
{ 0,-1, 0, 1, 0, 1, 0,-1, 0, 1, 0,-1, 0,-1, 0, 1 },
{ 0,-1, 0, 1, 0, 1, 0,-1,-1, 0, 1, 0, 1, 0,-1, 0 },
{ 0,-1,-1, 2, 0, 1,-1, 0, 1, 0, 0,-1, 1,-2, 0, 1 },
{ 0,-2, 1, 1, 2, 0,-1,-1, 0, 0,-1, 1, 0, 0,-1, 1 },
{ 0,-2, 0, 2, 1, 1,-1,-1, 0, 0, 0, 0, 1,-1,-1, 1 },
{-1, 2, 0,-1,-1, 0, 0, 1, 0,-1, 1, 0, 0, 1, 1,-2 },
{-1, 1, 1,-1, 0, 0, 0, 0, 1, 1,-1,-1,-2, 0, 2, 0 },
{-1, 1, 1,-1, 0, 0, 0, 0, 0, 0, 0, 0,-1, 1, 1,-1 },
{-1, 1, 1,-1, 0, 0, 0, 0,-1,-1, 1, 1, 0, 2, 0,-2 },
{-1, 1, 0, 0,-1, 1, 0, 0, 1, 1, 0,-2,-1,-1, 2, 0 },
{-1, 1, 0, 0,-1, 1, 0, 0, 0, 0, 1,-1, 0, 0, 1,-1 },
{-1, 1, 0, 0,-1, 1, 0, 0,-1,-1, 2, 0, 1, 1, 0,-2 },
{-1, 0, 2,-1, 1, 0, 0,-1, 0, 1,-1, 0,-2, 1, 1, 0 },
{-1, 0, 1, 0, 1, 0,-1, 0, 1, 0,-1, 0,-1, 0, 1, 0 },
{-1, 0, 1, 0, 0, 1, 0,-1, 0, 1, 0,-1,-1, 0, 1, 0 },
{-1, 0, 1, 0, 0, 1, 0,-1,-1, 0, 1, 0, 0, 1, 0,-1 },
{-1, 0, 0, 1, 1, 0,-2, 1, 2,-1,-1, 0, 0,-1, 1, 0 },
{-1, 0, 0, 1, 0, 1,-1, 0, 1, 0, 0,-1, 0,-1, 1, 0 },
{-1, 0, 0, 1, 0, 1,-1, 0, 0,-1, 1, 0, 1, 0, 0,-1 },
{-1, 0, 0, 1,-1, 2, 0,-1, 0, 1, 1,-2, 0,-1, 1, 0 },
{-1,-1, 2, 0, 1, 1, 0,-2,-1, 1, 0, 0,-1, 1, 0, 0 },
{-1,-1, 1, 1, 2, 0,-2, 0, 1,-1,-1, 1, 0, 0, 0, 0 },
{-1,-1, 1, 1, 1, 1,-1,-1, 0, 0, 0, 0, 0, 0, 0, 0 },
{-1,-1, 1, 1, 0, 2, 0,-2,-1, 1, 1,-1, 0, 0, 0, 0 },
{-1,-1, 0, 2, 1, 1,-2, 0, 1,-1, 0, 0, 1,-1, 0, 0 },
{-2, 1, 1, 0, 0, 1,-1, 0, 1, 0, 0,-1,-1, 0, 2,-1 },
{-2, 0, 2, 0, 1, 1,-1,-1, 0, 0, 0, 0,-1, 1, 1,-1 },
{-2, 0, 1, 1, 0, 2,-1,-1, 0, 0, 1,-1, 0, 0, 1,-1 }
	};

	int state[N] = {
	69, 31, 55, 149, 46, 43, 118, 30, 43, 45, 71, 380, 37, 36, 212, 52
	};
	double sum = 0.0;


	double mu[N] = {
63.12502, 44.15620, 49.96722, 146.75155, 33.08098, 48.63780, 120.01126, 35.26996,
49.99003, 48.73134, 74.91772, 365.36090, 33.03148, 29.24715, 226.87628, 47.84509
 	};
	double lratio;
	double tlratio;
        int tlratio_count = 0;

	if (argc >= 2) {
		seed = atoi(argv[1]);	
	} 
	if (argc >= 3) {
		n = atoi(argv[2]);	
	} 
	if (argc >= 4) {
		burnin = atoi(argv[3]);
	}

	lratio = likelihood_ratio(state, mu, N);
        printf("likelihood ratio statistic : %lf\n", lratio);

	srand(seed);

	/* burn-in */
	for (i = 0; i < burnin; i++) {
		j = (int)(M * (rand()/(RAND_MAX + 1.0)));
		r = prob_r(state, markov_b[j], N);	
		rr = (double)rand()/RAND_MAX;	
		if (r > rr)
			v_add(state, markov_b[j], N);
	}

	for (i = 0; i < n; i++) {
		j = (int)(M * (rand() / (RAND_MAX + 1.0))); 
		r = prob_r(state, markov_b[j], N);	
		rr = (double)rand()/RAND_MAX;	
		printf("p : %lf %lf\n", r, rr);
		if (r > rr)
			v_add(state, markov_b[j], N);
		tlratio = likelihood_ratio(state, mu, N);
		printf("%lf ", tlratio);
		if (tlratio > lratio) {
			tlratio_count++;
			printf("* ");
		}
		v_show(state, N);
	}
	printf("tlratio_count : %d\n", tlratio_count);
	printf("appoximate p-value : %lf\n", (double)tlratio_count/n);

	return 0;
}

