source("read_mar.r")
# マルコフ基底 
# 4ti2 のコマンド markov の出力結果とする
markov_b <- read_mar("covariate_mat2.mar")

# markov_b を -1 倍したものを付け加える
tm <- t(markov_b)
tmm <- -tm
markov_b <- matrix(append(tm, tmm), nrow(tm))
markov_b <- t(markov_b)

# p(y + z)/p(y) の計算 (z はマルコフ基底のいずれか)
# p(y) = 1/Z 1/(y[1]! ... y[length(y)]!)
prob_r <- function(y, z) {
	len <- length(y)
	r <- 1
	for (i in 1:len) {
		# マルコフ基底の成分が正, 負の場合で場合分けして計算
		if (z[i] > 0) {
			for (t in 1:z[i])
				r <- r / (y[i] + t)
		} else if (z[i] < 0) {
			for (t in 0:(z[i]+1))
				r <- r * (y[i] + t)
		}
	}
	return(r)
}

# 尤度比検定統計量の計算
my_likelihood_ratio <- function(x, fitted_v) {
	n <- length(x)
	sum <- 0
	for (i in 1:n) {
		sum <- sum + x[i] * log(x[i]/fitted_v[i])
	}
	sum <- 2 * sum
	return(sum)
}

test_my_likelihood_ratio <- function()
{
x <- c(69, 31, 55, 149, 46, 43, 118, 30, 43, 45, 71, 380, 37, 36, 212, 52)
fv <- c(64.52677, 47.25345, 53.14603, 151.07960, 30.42595, 46.79383, 115.24100, 32.53337,
        49.42430, 46.13193, 70.90290, 360.53502, 35.18867, 30.25510, 232.14438, 51.41770)
	return(my_likelihood_ratio(x, fv))
}

# 実験データの読み出し
dat <- read.table("2_7-3.dat", sep=",", header=T)
dat.glm <- glm(x~A+B+C+D+E+F+G+A*B+A*C+B*D, dat, family="poisson")
fv <- fitted(dat.glm)
fv <- as.vector(fv)
xx <- as.vector(t(dat["x"]))    

# mcmc 
# 引数
# n      : 取り出す数
# burnin : 最初からいくら捨てるか
# state  : 最初の状態 
cov2_mcmc <- function(n = 100000, burnin = 50000) {
	state <- xx
	lratio <- my_likelihood_ratio(state, fv)
	cat("likelihood ratio stat :\n")
	print(lratio)

	n_mar <- nrow(markov_b)
	select <- sample(c(1:n_mar), n, replace=TRUE) # マルコフ基底 markov_b のどれを選ぶか
	rr <- runif(n)
	for (i in 1:burnin) {
		index <- select[i]
		r <- prob_r(state, markov_b[index, ])	
		if (r > rr[i]) {
			state <- state + markov_b[index,]
		}
	}

	count <- 0
	select <- sample(c(1:n_mar), n, replace=TRUE) # マルコフ基底 markov_b のどれを選ぶか
	rr <- runif(n)
	for (i in 1:n) {
		index <- select[i]
		r <- prob_r(state, markov_b[index, ])	
		if (r > rr[i]) {
			state <- state + markov_b[index,]
		}
		tlratio <- my_likelihood_ratio(state, fv)
		if (tlratio > lratio) {
			count <- count + 1
			print("*")  # 条件に合うかどうか
		}
		print(state)        # サンプルの表示
		print(tlratio)      # そのサンプルの統計量
	}
	cat("n\n")
	print(n)
	cat("count\n")
	print(count)
	cat("p 値の近似値\n")
	print(count/n)
}

cat("cov2_mcmc(n, burnin) で実行\n")
cat("e.g. cov1_mcmc(10000, 10000)\n")

# 漸近的 p-value 0.004813
#> summary(dat.glm)
#
#Call:
#glm(formula = x ~ A + B + C + D + E + F + G + A * B + A * C +
#    B * D, family = "poisson", data = dat)
#
#Deviance Residuals:
#      1        2        3        4        5        6        7        8
# 0.7284  -2.0929   0.7005   0.1851   2.1196  -0.8248  -0.1841  -0.9110
#      9       10       11       12       13       14       15       16
#-1.0131  -0.5416  -0.4567   0.7608   0.6773   1.2047  -0.9987   0.5923
#
#Coefficients:
#            Estimate Std. Error z value Pr(>|z|)
#(Intercept)  4.17175    0.03392 122.995  < 2e-16 ***
#A           -0.10088    0.03290  -3.066  0.00217 **
#B            0.18430    0.03042   6.058 1.37e-09 ***
#C           -0.42330    0.03353 -12.625  < 2e-16 ***
#D            0.01489    0.03090   0.482  0.62992
#E            0.10488    0.03211   3.266  0.00109 **
#F           -0.02190    0.03101  -0.706  0.47990
#G            0.38059    0.03356  11.340  < 2e-16 ***
#A:B         -0.04693    0.03125  -1.502  0.13316
#A:C          0.18148    0.03318   5.470 4.51e-08 ***
#B:D         -0.29976    0.03380  -8.869  < 2e-16 ***
#---
#Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
#
#(Dispersion parameter for poisson family taken to be 1)
#
#    Null deviance: 1021.255  on 15  degrees of freedom
#Residual deviance:   16.838  on  5  degrees of freedom
#AIC: 135.01
#
#Number of Fisher Scoring iterations: 4
#
#> 1-pchisq(16.84, df=5)
#[1] 0.004813453
