#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define SSIZE 10000
#define BUCKET_SIZE 101 
#define QUEUE_N 1000000


typedef int *DATA;

typedef struct cell {
	DATA data;
	struct cell *next;
} CELL;

static vvsize = 0;

static DATA queue[QUEUE_N];
static int queue_top;
static int queue_tail;

static CELL *table[BUCKET_SIZE];

void set_vvsize(int n)
{
	vvsize = n;
}


int data_equal(DATA a, DATA b)
{
	int i;

	for (i = 0; i < vvsize; i++) 
		if (a[i] != b[i]) 
			return 0;
	return 1;
}

int data_hash(DATA a)
{
	int i;
	int val;

	val = 0;
	for (i = 0; i < vvsize; i++)
		val = (31 * val + a[i]);	
	if (val < 0)
		val = -val;
	return val;
}	

void data_show(DATA a)
{
	int i;

	for (i = 0; i < vvsize; i++)
		printf("%3d ", a[i]);	
}

void queue_init(int vvsize)
{
	set_vvsize(vvsize);

	queue_top = 0;
	queue_tail = 0;
}

int queue_put(DATA t)
{
	queue[queue_tail++] = t;
	if (queue_tail == queue_top)
		printf("queue is full\n");
	if (queue_tail == QUEUE_N)
		queue_tail = 0;
}

DATA queue_get()
{
	DATA t;

	if (queue_top == queue_tail)
		return NULL;
	t = queue[queue_top++];
	if (queue_top == QUEUE_N)
		queue_top = 0;
	return t;
}

int queue_empty()
{
	return queue_top == queue_tail;
}

int queue_size()
{
	if (queue_top <= queue_tail)
		return queue_tail - queue_top;
	return (QUEUE_N - queue_top) + queue_tail;
}


int hash(DATA t)
{
	return data_hash(t) % BUCKET_SIZE;
}

void show_hash_table()
{
	int i, n;
	CELL *p;

	for (i = 0; i < BUCKET_SIZE; i++) {
		for (n = 0, p = table[i]; p != NULL; n++, p = p->next)
			;
		printf("%3d : %3d\n", i, n);
	}
}

void init(int vvsize)
{
	int i;

	set_vvsize(vvsize);
	for (i = 0; i < BUCKET_SIZE; i++)
		table[i] = NULL;
}

int find(DATA t)
{
	CELL *p;

	for (p = table[hash(t)]; p != NULL; p = p->next) 
		if (data_equal(t, p->data))
			return 1;
	return 0;
}

int insert(DATA t)
{
	CELL *p;
	int h;

	if (find(t) == 1)
		return 0;
	if ((p = malloc(sizeof(CELL))) == NULL) {
		fprintf(stderr, "out of memory\n");
		exit(2);
	}
	h = hash(t);
	p->data = t;
	p->next = table[h];
	table[h] = p;
	return 1;
}

int delete(DATA t)
{
	int h;
	CELL *p, *q;
	
	h = hash(t);
	if (table[h] == NULL)
		return 0;
	if (data_equal(t, table[h]->data)) {
		p = table[h];
		table[h] = p->next;
		free(p);
		return 1;
	}
	for (q = table[h], p = q->next; p != NULL; q = p, p = p->next) {
		if (data_equal(t, p->data)) {
			q->next = p->next;
			free(p);
			return 1;
		}
	}
	return 0;
}

void show_vect(int *v, int n)
{
	int i;

	for (i = 0; i < n; i++)
		printf("%d ", v[i]);
	printf("\n");
}

int *new_vect(int *t, int n)
{
	int i;
	int *s;

	s = (int *)malloc(sizeof(int) * n);
	for (i = 0; i < n; i++)
		s[i] = t[i];
	return s;
}

int *add_vect(int *a, int *b, int n)
{
	int i;
	int *c;

	c = (int *)malloc(sizeof(int) * n);
	if (c == NULL) {
		fprintf(stderr, "run out of memory\n");	
		exit(2);
	}
	for (i = 0; i < n; i++)
		c[i] = a[i] + b[i];
	return c;
}

int *sub_vect(int *a, int *b, int n)
{
	int i;
	int *c;

	c = (int *)malloc(sizeof(int) * n);
	if (c == NULL) {
		fprintf(stderr, "run out of memory\n");	
		exit(2);
	}
	for (i = 0; i < n; i++)
		c[i] = a[i] - b[i];
	return c;
}

int test_add_nonnega(int *a, int *b, int *c, int n)
{		
	int i;
	for (i = 0; i < n; i++) {
		c[i] = a[i] + b[i];
		if (c[i] < 0)
			return 0;
	}
	return 1;
}

int test_sub_nonnega(int *a, int *b, int *c, int n)
{		
	int i;
	for (i = 0; i < n; i++) {
		c[i] = a[i] - b[i];
		if (c[i] < 0)
			return 0;
	}
	return 1;
}

int **read_data(char *filename, int *n, int *size)
{
	int i, j, k, len;
	int n_move, msize;
	char s[SSIZE];
	FILE *fp;
	int **move;
	char *format;
	int elem;

	if ((fp = fopen(filename, "r")) == NULL) {
		printf("can't open %s\n", filename);
		return 1;
	}

	if (fgets(s, SSIZE, fp) != NULL) {
		sscanf(s, "%d %d\n", &n_move, &msize);
		printf("n_move : %d, msize : %d\n", n_move, msize);
		move = (int **)malloc(sizeof(int *) * n_move);
		for (i = 0; i < n_move; i++)
			move[i] = (int *)malloc(sizeof(int) * msize);
	}
	i = 0;
	j = 0;
	while (fgets(s, SSIZE, fp) != NULL) {
		len = strlen(s);
		j = 0;
		for (k = 0; k < len; k++) {
			if (s[k] != ' ' && s[k] != '\n') {
				elem = atoi(s + k);
				move[i][j++] = elem;
				if (s[k] == '-')
					k++;
			}
		}
		i++;
	}
	fclose(fp);

	*n = n_move;
	*size = msize;
	return move;
}

int main(int argc, char *argv[])
{
	int tmp[SSIZE];
	int *s, *t;
	int i, j;
	int count, depth;
	int *result[1000000];
	int p = 0;

	int **start_v_a;
	int *start_v;
	int **move;
	int vsize, n_move, dummy;

	if (argc == 3) {
		start_v_a = read_data(argv[1], &dummy, &vsize);
		start_v = start_v_a[0];
		printf("start_v :\n");
		for (i = 0; i < vsize; i++) 
			printf("%2d ", start_v[i]);
		printf("\n");
		move = read_data(argv[2], &n_move, &vsize); 
		printf("move : %d %d\n", n_move, vsize);
		for (i = 0; i < n_move; i++) {
			for (j = 0; j < vsize; j++) 
				printf("%2d ", move[i][j]);
			printf("\n");
		}
	} else {
		printf("usage : %s start_file move_file\n", argv[0]);
		return 1;
	}

	init(vsize);
	queue_init(vsize);
	insert(start_v);
	queue_put(start_v);
	depth = 0;
	count = 1;
	result[p++] = start_v;
	while (!queue_empty()) {
		s = queue_get();
		for (i = 0; i < n_move; i++) {
			if (test_add_nonnega(s, move[i], tmp, vsize)) {
				if (!find(tmp)) {
					t = new_vect(tmp, vsize);
					insert(t);
					queue_put(t);
					count++;
					show_vect(t, vsize);
					result[p++] = t; 
				}
			}
		}
		for (i = 0; i < n_move; i++) {
			if (test_sub_nonnega(s, move[i], tmp, vsize)) {
				if (!find(tmp)) {
					t = new_vect(tmp, vsize);
					insert(t);
					queue_put(t);
					count++;
					show_vect(t, vsize);
					result[p++] = t; 
				}
			}
		}
		printf("depth : %d, count : %d, queue : %d\n", depth, count, queue_size());
		depth++;
	}
	printf("count : %d\n", count);

	printf("fiber :\n");
	for (i = 0; i < p; i++) {
		show_vect(result[i], vsize);
	}
}

